#! /bin/sh
LicenseFile=None;export LicenseFile
#
PortID=linux_x86-64;export PortID
#
ExtDB=psql10;export ExtDB
#
#Everything Above Here Generated By bldinst.sh
#
#   Copyright 1988, 1989, 1990, 1991,    DNA Data Systems and Kevin English
#   Copyright 1991 - 1996                4C Associates
# 
#	@(#) - $Id: 4cinst-extdb.sh,v 1.2 2019/04/04 19:09:04 kjenglish Exp $
#
#set -x
#
MyName=$0
HomeDir="$HOME_4C"
FCINSTHIST=/dev/tty
TopDir=`pwd`
TmpDir=$TopDir/4cTmp$$
ODev=/dev/null
export ODev
trap catchhup 1
trap catchint 2
trap catchquit 3
trap catchterm 15
#
# Figure out the echo with & without newline command
#
eval print 1 >/dev/null 2>&1 &&
{
	PRINT=print
} ||
{
	PRINT=echo
}
$PRINT -n | grep -- "-n" &&
{
	PRINTN="$PRINT"
} ||
{
	PRINTN="$PRINT -n"
}
usage()
{
	$PRINT "Usage: $MyName [ options ... ]"
	$PRINT "	Options:"
	$PRINT "		-help"
	$PRINT "		-home <homedir>"
	exit 1
}
cleanup()
{
	if [ `basename "$TmpDir"` = "4cTmp$$" ]
	then
		rm -rf $TmpDir
	fi
}
instexit()
{
	ExitCode=$1
	if [ "$ExitCode" = "0" ]
	then
		$PRINT "	OK" >>$FCINSTHIST
	else
		$PRINT "	ERROR - $ExitCode" >>$FCINSTHIST
	fi
	$PRINT "Done - `date \"+%d %h %Y - %T\"`" >>$FCINSTHIST
	cleanup
	exit $ExitCode
}
catchhup()
{
	trap 1
	instexit 129
}
catchint()
{
	trap 2
	instexit 130
}
catchquit()
{
	trap 3
	instexit 131
}
catchterm()
{
	trap 15
	instexit 143
}
warn()
{
	$PRINT "	WARNING: $1"
	$PRINT "WARNING - $1" >> $TmpDir/Warn
	$PRINT "	WARNING - $1" >>$FCINSTHIST
}
fail()
{
	$PRINT "	FATAL ERROR: $1"
	$PRINT "FATAL ERROR - $1" >> $TmpDir/Fail
	$PRINT "	FATAL ERROR - $1" >> $FCINSTHIST
	instexit 1
}
#
# get yes/no answer
#
yn()
{
	Prompt=$1
	while $PRINTN "$Prompt? (y/n/Q) "
	do
		Answ=""
		read Answ
		if [ "$Answ" = "y" ]
		then
			return 0
		fi
		if [ "$Answ" = "n" ]
		then
			return 1
		fi
		if [ "$Answ" = "Q" ]
		then
			instexit 1
		fi
		$PRINT "Must Enter 'y', 'n', or 'Q' only"
	done
	return 1
}
#
# Only prompt if full install not specified
#
instyn()
{
	if [ "$FullInst" = "Yes" ]
	then
		return 0
	fi
	yn "$@"
}
verifydir()
{
	if [ `dirname $1` = '.' ]
	then
		$PRINT "Full Pathname Required"
		return 1
	fi
	if [ ! -r "$TopDir/4cmkdirs.sh" ]
	then
		$PRINT "Non executable 4cmkdirs.sh"
		return 1
	fi
	/bin/sh $TopDir/4cmkdirs.sh "$1" || return 1
	if [ -z "$FCGroup" -o -z "$FCOwner" -o -z "$DirMode" ]
	then
		:
	else
		chgrp $FCGroup $1 2>/dev/null
		chown $FCOwner $1 2>/dev/null
		chmod $DirMode $1
	fi
	if [ ! -w  "$1" ]
	then
		$PRINT "Directory "$1" Must be writable for Install"
		return 1
	fi
	return 0
}
readinfo()
{
	if [ ! -f "$FCINFO" ]
	then
		$PRINT "Cannot read file: $FCINFO"
		return 1
	fi
	if [ ! -r "$FCINFO" ]
	then
		$PRINT "Cannot read file: $FCINFO"
		return 1
	fi
	exec 3< $FCINFO
	while read Reply 0<&3
	do
		set $Reply
		case $1 in
			    Owner)	FCOwner="$2"
						;;
			     Group)	FCGroup="$2"
						;;
			  base_dir) FCBaseDir="$2"
					;;
			   bin_dir) FCBinDir="$2"
					;;
			   doc_dir) FCDocDir="$2"
					;;
			   etc_dir) FCEtcDir="$2"
					;;
			   lib_dir) FCLibDir="$2"
					;;
			   samples_dir) FCSampDir="$2"
					;;
			  security) FCSecurity="$2"
					;;
			system_dir) FCSysDir="$2"
					;;
			tutorial_dir) FCTutDir="$2"
					;;
			demo_dir) FCDemoDir="$2"
					;;
			serial_dir) #ignore
					;;
			image_dir) FCImageDir="$2"
					;;
			         *)	$PRINT "Illegal Line in $FCINFO $1"
					;;
		esac
	done
	exec 3<&-
	case $FCSecurity in
			owner)	EXMode="a-rwx,u=x"
					DirMode="a-rwx,u=rwx"
					SHMode="a-rwx,u=rx"
					BootMode="a-rwx,u=rw"
					ROMode="a-rwx,u=r"
					;;
			none)   EXMode="a=rwx"
					DirMode="a=rwx"
					SHMode="a=rwx"
					BootMode="a=rw"
					ROMode="a=r"
					;;
			 group|*)	EXMode="a-rwx,g=x,u=x"
						DirMode="a-rwx,g=rwx,u=rwx"
						SHMode="a-rwx,g=rx,u=rx"
						BootMode="a-rwx,g=rw,u=rw"
						ROMode="a-rwx,g=r,u=r"
					;;
	esac
	return 0
}
#
# Process args
#
while test "$#" != "0"
do
	arg=$1
	shift 1
	case $arg in
		-help|-h|-H)	usage ;;
		-home)	if test "$#" = 0
				then
					$PRINT "Must Specify Directory Name with -home"
					usage
				fi
				HomeDir=$1;shift 1;;
		*)		echo "Illegal arg: $arg"
				usage;;
	esac
done
if [ -z "$LOGNAME" ]
then
	for X in `who am i`
	do
		LOGNAME="$X"
		break
	done
fi
#
if [ -z "$HomeDir" ]
then
	HOME_4C="/usr/4c"
	$PRINT "4C Home Directory Not Set or Specified"
	yn "OK to use \"$HOME_4C\" " ||
		{ echo "Please set HOME_4C or use -home option";
			usage; }
	HomeDir=$HOME_4C
else
	if [ `dirname $HomeDir` = "." ]
	then
		$PRINT "Invalid Home Dir for 4C: $HomeDir"
		$PRINT "Full Pathname required for 4C Home Directory"
		usage
	fi
fi
verifydir "$HomeDir" || exit 1
FCINFO="$HomeDir/4C.info"
if [ -z "$PortID" ]
then
	$PRINT "Must Set PortID"
	instexit 1
fi
if [ -z "$ExtDB" ]
then
	$PRINT "Must Set ExtDB"
	instexit 1
fi
if [ -d $TmpDir ]
then
	rm -rf $TmpDir
fi
mkdir $TmpDir  || instexit 1
if [ -z "$GRPNAME" ]
then
	>$TmpDir/X$$
	GRPNAME=`ls -l $TmpDir/X$$ |
	awk '{ \
			print $4; \
	}'`
fi
PATH=$TopDir:$PATH
	export PATH
readinfo || instexit 1
FCINSTHIST=$FCEtcDir/4Cinst.hist
date "+%d %h %Y - %T - $LOGNAME" >>$FCINSTHIST
#
# Re verify homedir to set permissions
#
verifydir "$HomeDir" || instexit 1
#
# Install All Files in Dir Skipping or Recursing into subdirectories
#
DirInstall()
{
	verifydir $2
	if [ ! -d $1 ]
	then
		return 0
	fi
	MyMode=$3
	if [ "$4" = "Recurse" ]
	then
		Recurse="Yes"
	fi
	(
	cd $1
#
# Subdirs first
#
	if [ "$Recurse" = "Yes" ]
	then
		for FileName in *
		do
			if [ -d "$FileName" ]
			then
				DirInstall "$FileName" "$2/$FileName" "$MyMode" "Recurse"
			fi
		done
	fi
#
# Any license file will be a .txt file
#
	for TextFile in *.txt
	do
		if [ "$TextFile" = "*.txt" ]
		then
			break
		fi
		rm -f $2/$TextFile
		cp $TextFile $2/$TextFile
		chgrp $FCGroup $2/$TextFile 2>/dev/null
		chown $FCOwner $2/$TextFile 2>/dev/null
		chmod $ROMode $2/$TextFile
	done
	for GZFile in *.gz
	do
		if [ "$GZFile" = "*.gz" ]
		then
			break
		fi
		BootFile=`expr $GZFile : '\(.*\).gz'`
		$PRINTN "$GZFile ==> $2/$BootFile - "
		rm -f $2/$BootFile
		gzip -d -c $GZFile >$2/$BootFile ||
			{
				$PRINT "Failed";
				warn "Copy/Uncompress Failed - $BootFile";
				yn "Continue" || instexit 1
				continue;
			}
		if [ -f $BootFile.sum ]
		then
			rm -f $TmpDir/$BootFile$$
			sum $2/$BootFile |
				sed 's/ [^ ]*$//' >$TmpDir/$BootFile$$
			cmp -s $TmpDir/$BootFile$$ $BootFile.sum && 
					$PRINT "OK" ||
					{ $PRINT "Failed"; warn "Checksum Failed - $BootFile"; }
			rm -f $TmpDir/$BootFile$$
		fi
		chgrp $FCGroup $2/$BootFile 2>/dev/null
		chown $FCOwner $2/$BootFile 2>/dev/null
		chmod $MyMode $2/$BootFile
	done
	) || exit $?
}
#
# Force acceptance of license if it exists
if [ ! "$LicenseFile" = "None" ]
then
	echo ""
	cat lib/${LicenseFile}
	echo ""
	yn "Accept License" || exit 1
fi
#
yn "Full Install" && FullInst="Yes"
#
#
instyn "Install Lib Files" &&
{
	$PRINT "	Installing $ExtDB Lib" >>$FCINSTHIST
	if [ -f "$TopDir/4CExtDB.ver" ]
	then
		cat $TopDir/4CExtDB.ver >>$FCINSTHIST
	fi
	$PRINT "Copying Lib Files"
	DirInstall $TopDir/lib $FCLibDir $SHMode
#
# Set up Symbolic Links to Shared Libraries
#
	cd $FCLibDir
	if [ "$PortID" = hp9000 ]
	then
		SHLIBEXT="sl"
	else
		SHLIBEXT="so"
	fi
	rm -f lib4c${ExtDB}.${SHLIBEXT}
	ln -s rellib4c${ExtDB}.${SHLIBEXT} lib4c${ExtDB}.${SHLIBEXT}
}
instexit 0
